%{
/* Konfigurationsdateiinterpreter mit lex und yacc
   Lexer

											Datei: cfgdatei.l
											Datum: 26.11.1995
											Rev. : 14.10.1996
											Autor: KS
*/

#include <string.h>

#if ((!defined _INCLUDE_LEX_YACC_) && (!defined __CFGDATEI_H_INCLUDED__))
 /* Extradefinitionen, wenn jedes Modul einzeln bersetzt wird */
#include "cfgdat_y.h"
#include "cfgdatei.h"

extern INHALT *yyinhalt;
extern int yyzeile;
extern char *yycfgdatei;
extern int yyabschnitt;
#endif
%}

ws         [ \t]+
kommentar  (#|"//").*
qstring    \"[^\"\n]*[\"\n]
nummer     [0-9]+
abschnitt  "["[^\]\n]*"]"
bezeichner [a-zA-Z][a-zA-Z0-9]*
nl         \n
%%

{ws} ;        /* Leerzeichen und Tabs berlesen */
{kommentar} ; /* Kommentare berlesen */

{abschnitt} { /* Abschnitt */
	yylval.string = yytext;
	return ABSCHNITT;
}

{nummer} { /* Nummer */
	yylval.val = atoi(yytext);
	return NUMBER;
}

{qstring} { /* mit doppelten Anfhrungszeichen eingeschlossener String */
	if ((yylval.string = strdup(yytext+1)) == NULL) { /* Anfngliches '"' nicht mit */
		yyerror(F_SPEICHER);
		return 0; /* Abbruch */
	}
	if (*(yylval.string+yyleng-2) != '"') { /* Kein '"' am Ende */
		warning("Unbeendeter String");
	}
	else {
		*(yylval.string+yyleng-2) = '\0'; /* '"' am Ende entfernen */
	}
	return STRING;
}

{bezeichner} { /* Bezeichner */
	if ((yylval.string = strdup(yytext)) == NULL) { /* Man darf den yytext - String ja nicht benutzen */
		yyerror(F_SPEICHER);
		return 0; /* Abbruch */
	}
	return BEZEICHNER;
}

{nl} { /* Neue Zeile */
	yyzeile++;
}

. { /* Alles restliche */
	return *yytext;
}

%%
void cfgdateilesen(FILE *fp,char *cfgdatei,INHALT *inhalt)
{
	yyin = fp;
	yyinhalt = inhalt;
	yyzeile = 1;
	yycfgdatei = cfgdatei;
	if (inhalt[0].abschnitt.abschnitt == NULL) {
		/* Erster Inhalt = Kein Abschnitt (vor dem ersten) */
		yyabschnitt = 0;
		/* Damit es keinen Memory-fault gibt */
		inhalt[0].abschnitt.abschnitt = "";
	}

	yyparse();

	/* Lexer wieder in den Anfangszustand bringen */
	BEGIN INITIAL;
	YY_NEW_FILE;

	return;
}

#ifdef ZEILEN_LESEN
void cfgdateizeilelesen(FILE *fp, char *cfgdatei, char *abschnitt, char *bezeichner,char **wert, int flags)
{
	INHALT inhalt[] = { { { NULL }, 0, NULL },
	                    { { NULL }, 0, NULL } } ;
	EINTRAEGE eintrag[] = { { NULL, 0, NULL, NULL },
	                        { NULL, 0, NULL, NULL } };

	if (abschnitt == NULL) {
		yyabschnitt = 0;
		abschnitt = "";
	}

	inhalt[0].abschnitt.abschnitt = abschnitt;
	inhalt[0].eintraege = eintrag;
	inhalt[0].optionen  = (unsigned char)flags;
	eintrag[0].bezeichner = bezeichner;
	eintrag[0].werte.string_ptr = wert;
	eintrag[0].typ = ((flags & BEZEICHNER) || (flags & BEZEICHNER_PTR)) ? BEZEICHNER_PTR : STRING_PTR;

	yyin = fp;
	yyinhalt = inhalt;
	yyzeile = 1;
	yycfgdatei = cfgdatei;

	yyparse();

	/* Lexer wieder in den Anfangszustand bringen */
	BEGIN INITIAL;
	YY_NEW_FILE;

	return;
}
#endif

#ifndef EXTERN_YYERROR_FUNK
void yyerror(char *s)
{
	#ifdef __YY_EXT_YYERROR__
	__YY_EXT_YYERROR__
	#else
	fprintf(stderr,"Fehler: %s bei \"%s\" in Zeile %d von '%s'\n",s,yytext,yyzeile,yycfgdatei);
	#endif
	return;
}

void yyexterror(char *s,char *t)
{
	#ifdef __YY_EXT_YYEXTERROR__
	__YY_EXT_YYEXTERROR__
	#else
	fprintf(stderr,"Fehler: %s '%s' in Zeile %d von '%s'\n",s,t,yyzeile,yycfgdatei);
	#endif
	return;
}

void warning(char *s)
{
	#ifdef __YY_EXT_WARNING__
	__YY_EXT_WARNING__
	#else
	fprintf(stderr,"Warnung: %s bei \"%s\" in Zeile %d von '%s'\n",s,yytext,yyzeile,yycfgdatei);
	#endif
	return;
}
#endif
