%{
/* Konfigurationsdateiinterpreter mit lex und yacc
   Parser

											Datei: cfgdatei.y
											Datum: 26.11.1995
											Rev. : 01.10.1996
											Autor: KS
*/

#include <stdio.h>

#if ((!defined _INCLUDE_LEX_YACC_) && (!defined __CFGDATEI_H_INCLUDED__))
 /* Extradefinitionen, wenn jedes Modul einzeln bersetzt wird */
#include "cfgdatei.h"
#endif

/* Rckgabewerte von keyword() und abschnitte() im Falle eines Fehlers */
#define NICHT_GEFUNDEN -1
#define DONT_CARE      -2
#define FEHLER         -3

/* Globale Variablen */
INHALT *yyinhalt; /* Inhalt */
int yyzeile;      /* Zeilennummer */
char *yycfgdatei; /* Konfigurationsdateiname */
int yyabschnitt = NICHT_GEFUNDEN; /* Aktueller Abschnitt */
%}

%union {
	long val;
	char *string;
}

/* Werte */
%token <string> STRING;         /* String */
%token <string> STRING_PTR;     /* Zeiger auf String */
%token <val>    NUMBER;         /* Nummer */
%token <val>    INT_PTR;        /* Zeiger auf Integer */
%token <val>    LONG_PTR;       /* Zeiger auf Long */
%token <string> BEZEICHNER;     /* Bezeichnerstring */
%token <string> BEZEICHNER_PTR; /* Zeiger auf Bezeichnerstring */
%token <val>    DEF_BEZEICHNER; /* Bezeichner mit bestimmten int-Wert */
%token <string> ABSCHNITT;      /* Abschnitt */
%%

statement_list: /* Nichts */
	|	statement
	|	statement_list statement
	;

statement: BEZEICHNER '=' STRING ';' {
			int stelle;

			switch(keyword($1,&stelle)) { /* Welcher Typ? */
				case EGAL: /* Typ egal */
				case STRING:
					yyinhalt[yyabschnitt].eintraege[stelle].werte.string = $3;
					yyinhalt[yyabschnitt].eintraege[stelle].typ = STRING;
					break;
				case STRING_PTR:
					*(yyinhalt[yyabschnitt].eintraege[stelle].werte.string_ptr) = $3;
					yyinhalt[yyabschnitt].eintraege[stelle].typ = STRING_PTR;
					break;
				case NICHT_GEFUNDEN:
					yyexterror(F_BEZEICHNER,$1);
					free($3); /* Wert wird nicht mehr benutzt */
					break;
				case DONT_CARE:
					free($3); /* Wert wird nicht mehr benutzt */
					break; /* Nichts */
				default: /* Falscher Typ */
					yyexterror(F_TYP,$1);
					free($3); /* Wert wird nicht mehr benutzt */
					break;
			}
			free($1); /* Bezeichner wird nicht mehr gebraucht */
		}

	|	BEZEICHNER '=' NUMBER ';' {
			int stelle;

			switch (keyword($1,&stelle)) {
				case EGAL: /* Typ egal */
				case NUMBER:
					yyinhalt[yyabschnitt].eintraege[stelle].werte.val = (long)$3;
					yyinhalt[yyabschnitt].eintraege[stelle].typ = NUMBER;
					break;
				case INT_PTR:
					*(yyinhalt[yyabschnitt].eintraege[stelle].werte.int_ptr) = (int)$3;
					yyinhalt[yyabschnitt].eintraege[stelle].typ = INT_PTR;
					break;
				case LONG_PTR:
					*(yyinhalt[yyabschnitt].eintraege[stelle].werte.long_ptr) = (long)$3;
					yyinhalt[yyabschnitt].eintraege[stelle].typ = LONG_PTR;
					break;
				case NICHT_GEFUNDEN:
					yyexterror(F_BEZEICHNER,$1);
					break;
				case DONT_CARE:
					break; /* Nichts */
				default:
					yyexterror(F_TYP,$1);
					break;
			}
			free($1); /* Bezeichner wird nicht mehr gebraucht */
		}

	|	BEZEICHNER '=' BEZEICHNER ';' {
			int stelle,zw;

			switch(keyword($1,&stelle)) {
				case EGAL: /* Typ egal */
				case BEZEICHNER:
					yyinhalt[yyabschnitt].eintraege[stelle].werte.string = $3;
					yyinhalt[yyabschnitt].eintraege[stelle].typ = BEZEICHNER;
					break;
				case BEZEICHNER_PTR:
					*(yyinhalt[yyabschnitt].eintraege[stelle].werte.string_ptr) = $3;
					yyinhalt[yyabschnitt].eintraege[stelle].typ = BEZEICHNER_PTR;
					break;
				case DEF_BEZEICHNER:
					if (bezeichner_def(yyinhalt[yyabschnitt].eintraege[stelle].def,$3,&zw) != NICHT_GEFUNDEN) {
						yyinhalt[yyabschnitt].eintraege[stelle].werte.val = zw;
						yyinhalt[yyabschnitt].eintraege[stelle].typ = DEF_BEZEICHNER;
					}
					else {
						yyexterror(F_BEZ_WERT,$3);
					}
					free($3);
					break;
				case NICHT_GEFUNDEN:
					yyexterror(F_BEZEICHNER,$1);
					free($3); /* Wert wird nicht mehr benutzt */
					break;
				case DONT_CARE:
					free($3); /* Wert wird nicht mehr benutzt */
					break; /* Nichts */
				default:
					yyexterror(F_TYP,$1);
					free($3); /* Wert wird nicht mehr benutzt */
					break;
			}
			free($1); /* Bezeichner wird nicht mehr gebraucht */
		}

	|	ABSCHNITT {
			int zw;
			INHALT *inhptr = yyinhalt;

			switch(zw = abschnitte($1)) {
				case NICHT_GEFUNDEN:
					yyexterror(F_ABSCHNITT,$1);
					break;
				default: /* Gut gegangen */
					yyabschnitt = zw;
					break;
			}

			for(;inhptr->eintraege != NULL;inhptr++); /* Ans Ende der Inhaltsstruktur gehen */
			if (inhptr->optionen & ABSCHNITTE) {
				/* Man will eine Liste aller Abschnitte haben */
				ALLE_ABSCHNITTE *abptr,*abptr_vor = NULL;
				int len = strlen($1)-2; /* Lnge des Abschnittstrings */

				abptr = inhptr->abschnitt.alle_abschnitte; /* Hier werden alle Abschnittsnamen gespeichert */
				for(;abptr;abptr_vor = abptr,abptr = abptr->next); /* Ans Ende gehen */

				#ifdef __EIN_FELD__
				if ((abptr = (ALLE_ABSCHNITTE *)malloc(sizeof(ALLE_ABSCHNITTE)+len+1)) == NULL) {
				#else
				if (((abptr = (ALLE_ABSCHNITTE *)malloc(sizeof(ALLE_ABSCHNITTE))) == NULL) || ((abptr->abschnitt = (unsigned char *)malloc(len+1)) == NULL)) {
				#endif

					yyerror(F_SPEICHER);
					return 1; /* Abort */

				#ifdef __EIN_FELD__
				}
				abptr->abschnitt = (char *)(abptr+1);
				#else
				}
				#endif

				abptr->next = NULL;
				strncpy(abptr->abschnitt,$1+1,len);
				*(abptr->abschnitt+len) = '\0';
				if (!abptr_vor) /* Der allererste Eintrag */
					inhptr->abschnitt.alle_abschnitte = abptr;
				else /* Alle weiteren Eintrge */
					abptr_vor->next = abptr;
			}
		}
	;
%%
int keyword(char *word,int *stelle_ind)
{
	INHALT *ptr = yyinhalt;

	if (yyabschnitt < 0) /* Falscher Abschnitt */
		return yyabschnitt;

	for ((*stelle_ind)=0;ptr[yyabschnitt].eintraege[(*stelle_ind)].bezeichner != NULL;(*stelle_ind)++) { /* Alle Eintrge eines Abschnitts durchlaufen */
		if (!stringcompare(ptr[yyabschnitt].eintraege[(*stelle_ind)].bezeichner,word)) { /* Richtiger Eintrag ? */
			/* Ja, Treffer ! */
			return ptr[yyabschnitt].eintraege[(*stelle_ind)].typ;
		}
	}
	/* Nicht gefunden */
	return (ptr[yyabschnitt].optionen & MEHR) ? DONT_CARE : NICHT_GEFUNDEN;
}

int abschnitte(char *word)
{
	INHALT *ptr = yyinhalt;
	int ind,
	    len = strlen(word)-2;

	for(ind=0;ptr[ind].eintraege != NULL;ind++) {
		if (!strncmp(ptr[ind].abschnitt.abschnitt,(word+1),len)) {
			return ind;
		}
	}
	/* Nicht gefunden */
	return (ptr[ind].optionen & MEHR) ? DONT_CARE : NICHT_GEFUNDEN;
}

int bezeichner_def(BEZEICHNER_DEF *def,char *word,int *wert)
{
	int ind;

	if (!def)
		return NICHT_GEFUNDEN; /* Null-Zeiger */

	for (ind=0;def[ind].bezeichner;ind++) {
		if (!stringcompare(def[ind].bezeichner,word)) {
			*wert = def[ind].wert;
			return ind;
		}
	}
	return NICHT_GEFUNDEN;
}

int stringcompare(char *string1, char *string2)
{
	if (yyinhalt[yyabschnitt].optionen & GROSSKLEINEGAL)
		return stricmp(string1,string2);
	else
		return strcmp(string1,string2);
}
