/* Header fuer KonnMuck

										Datei: header.h
										Datum: 01.09.1996
										Rev. : 28.12.1997
										Rev.2: 30.04.2000
										Autor: KS
*/

#ifndef __HEADER_H_INCLUDED__
#define __HEADER_H_INCLUDED__


/* Fr alle Programme ----------------------------------------------------- */

/* Meldungsaustausch */

/* Meldungstypen */
#define KM_OPENSOCK  2 /* open_con -> receive : Socket geffnet. Parameter: int sock; */
#define KM_CLOSESOCK 3 /* receive -> ausgabe, close_con, quit: Socket geschlossen */
#define KM_STRING    4 /* receive -> ausgabe: String gesendet. Parameter: char string[]; */
#define KM_MELDUNG   5 /* connect -> ausgabe: Statusmeldung. Parameter: char string[]; */
#define KM_TEST      6 /* init -> receive, connect: Programm verfgbar? */

#define KM_IST_OKAY  0 /* Antwort: OK */
#define KM_FEHLER   -1 /* Antwort: FEHLER */

/* Die Kennung */
#define KENNUNG "KonnMuck"
#define KENNUNGSLEN 9


/* Die Meldungsstruktur */
#define IMMER_IN_MSG /* Was in jeder Meldung ist */ \
	char kennung[KENNUNGSLEN]; /* Kennung */ \
	signed char typ /* Typ */ /* Kein Semikolon */

typedef struct {
	IMMER_IN_MSG; /* Normales */
} MELDUNG;

typedef struct {
	IMMER_IN_MSG; /* Normales */
	char string[];
} STRING_MELDUNG;

typedef struct {
	IMMER_IN_MSG;
	int val;       /* Fehler- bzw, Meldungnummer (z.B. IV_MSG_CONNECT) */
	int v_errno;   /* errno */
	char string[]; /* zustzliche Info im String */
} MSG_MELDUNG;

typedef struct { /* Meldungen open_con -> connect */
	IMMER_IN_MSG; /* Normales */
	int sock;               /* Socket */
	int hostaddroffset;     /* Offset zu hp->h_addr */
	int h_addrtype;         /* hp->h_addrtype */
	int h_length;           /* hp->h_length */
	int port;               /* Port */
	int hostnamenoffset;    /* Offset zu hp->h_name */
	int namenoffset;        /* Offset zum Charakternamen */
	int passwdoffset;       /* Offset zum Passwort */
	/* String, in dem die ganzen Strings enthalten sind */
	char string[];
} CONNECT_MELDUNG;

#undef KENNUNGSLEN

/* Meldungsstruktur initialisieren */
#define INIT_MELDUNG(__msg,__typ) \
	strcpy((__msg)->kennung,KENNUNG); \
	(__msg)->typ = __typ;

/* Antwort OK? */
#define OKAY(__msg) ((!strcmp((__msg)->kennung,KENNUNG)) && ((__msg)->typ == KM_IST_OKAY))

/* Kennung OK? */
#define KENNUNG_OKAY(__msg_ptr) (!strcmp((__msg_ptr)->kennung,KENNUNG))

/* Name des Empfangsprogramms */
#define EMPFANGS_FUNK "konnreceive"

/* Name des Programms, das die Verbindung aufbaut */
#define CONNECT_FUNK "konnconnect"

/* Fehler- usw. Meldungen */
#include "msg.h"



#ifdef __RECEIVE_PROC__
/* Nur fr das Empfangsprogramm ------------------------------------------- */

#define BUFLEN 512 /* 0,5 KB Ausgabepuffer */

#elif (defined __CONNECT_PROC__)
/* Nur fr das Verbindungsprogramm ---------------------------------------- */

#define BUFLEN 512 /* 0,5 KB */

#else
/* Nur fr das Haupt-KonnMuck-Programm ------------------------------------ */

/* Konstanten */
#define BUFLEN      2048   /* Lnge des Universalpuffers (2KB) */
#define PASSWDLEN   32     /* Maximale Lnge der Passwrter (mit '\0') */
#define AUSGENDE    0xFFFF /* Wert des "end"-Parameters zum Lschen des gesamten Textes im Ausgabe-Widget */

/* Fr do_command(): Werte des mode - Argumentes */
#define DC_NICHTS  0
#define DC_AUSGABE 1

/* Fr AddString() */
#define SCHRIFTFLAG 0x03 /* Wo die Schriftinfo steht */
#define NORMAL      0x01
#define LOKAL       0x02
#define KOMMANDO    0x03

/* Datendatei */
#define DATENDATEI ".konnmuck.data"


/* Typedefs */
#define __HEADER_H_DA__ /* Damit config.h wei, da diese Stelle hier da ist */
#include "config.h" /* Deklarationen: 
		typedef {...} KONFIG;
		typedef {...} MUCKDATEN;
		typedef {...} MUCKS;
*/
#undef __HEADER_H_DA__ /* Stelle zu Ende */

typedef struct {
	char *empfangs_pfad; /* Pfad und Name des Empfangsprozesses */
	pid_t empfangs_pid;  /* Id des Empfangsprozesses */
	char *connect_pfad;  /* Pfad und Name des Connectprozesses */
	pid_t connect_pid;   /* Id des Connectprozesses */
	PtInputId_t *ipid;   /* Id der Empfangsmeldungsverarbeitungsfunktion */
} PROZESSDATEN;

typedef struct {
	int sock;  /* Socket; -1 = nicht verbunden */
	FILE *fpLog; /* Dateizeiger zur Logdatei, NULL wenn kein Log */
} VERBINDUNG;


/* Globale Variablen */

#ifdef __MAIN__ /* Variablen nur einmal normal, sonst immer als extern definieren */
#define ISEXTERN
#else
#define ISEXTERN extern
#endif

ISEXTERN KONFIG konfigdaten;
ISEXTERN MUCKS  mucks;

ISEXTERN VERBINDUNG verbindung; /* Daten ber die aktuelle Verbindung */
ISEXTERN unsigned char *buf; /* Puffer */

ISEXTERN PROZESSDATEN prozessdaten;

ISEXTERN char *muckpasswort;

ISEXTERN ApDBase_t *textdb; /* Fr Mehrsprachen-Untersttzung */


/* Funktionsprototypen */
/* ausgabe.c */
extern int ausgabe(void *,pid_t,void *,size_t);
extern int AddString(char *,unsigned char);
extern void log_command(char *);
extern void do_connect(char *);
extern void do_disconnect(void);
/* open_con.c */
int open_connection(char *,int,char *,char *);
/* send_command.c */
extern void do_command(char *,unsigned int);
/* winmsg.c */
extern int close_msg(void);
extern int resize_msg(PhDim_t);
extern int move_msg(PhPoint_t);
/* close_con.c */
extern void prozesse_runterfahren(void);
/* init.c */
int read_configfile(void);
void read_datafile(void);
void write_datafile(void);
/* about.c */
void get_version(char *,char *);
/* cfgdatei.c */
int read_configfile(void);
FILE *make_sample_cfgdatei(char *);


#undef ISEXTERN
#endif /* #else [von __RECEIVE_PROC__] */

#endif /* __HEADER_H_INCLUDED__ */
