/* Kontrolle der Eingabe des Passwortes im Textwidget
   Verndert bernommen aus QUICS:

 * Example code snippet showing the use of a CB_MODIFY_VERIFY callback
 * on a PtText widget to implement a password entry field, where input
 * characters are echoed as '*'. Full editing is supported (although of
 * dubious use, the user may use arrows/mouse to select/delete/modify
 * any portion of the password).


										Datei: passweing.c
										Datum: 14.10.1996
										Rev. : 07.12.1996
										Autor: KS

rcs_id: $Id: passweing.c,v 3.1 1996/12/07 20:13:10 klaus Exp klaus $

                            AppBuilder Photon Code Lib
                                         Version 1.11B
*/

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "header.h"
#include "abimport.h"
#include "proto.h"


/*
 * Callback function for the password PtText field, which echos characters
 * as '*'. The real password is stashed away in another string (user data).
 * When a CB_MODIFY_VERIFY is called, the function is given a PtTextCallback
 * structure, which contains the following members:
 *   start_pos, end_pos: characters to be deleted from the string
 *   length: length of characters to be inserted into string
 *   cur_insert: position to insert those characters
 *   text: the actual characters being inserted
 *   doit: a flag that this function may clear to disallow the action
 */
int passweing(PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo)
{
	PtTextCallback_t	*txt;
	int					plen;
	char *passwd = muckpasswort;

	// Get the text callback information structure
	txt = (PtTextCallback_t *)cbinfo->cbdata;

	// Get the length of the current plain-text password
	plen = strlen(passwd);

	// If start_pos != end_pos then some characters are being deleted
	// Simply shuffle the remaining password characters along
	if (txt->start_pos > txt->end_pos)
		memmove(&passwd[txt->end_pos], &passwd[txt->start_pos], plen - txt->start_pos + 1);
	else if (txt->start_pos < txt->end_pos)
		memmove(&passwd[txt->start_pos], &passwd[txt->end_pos], plen - txt->end_pos + 1);

	// If length != 0 then some characters are being inserted at cur_insert
	// Make room for them at that position, and copy into the password
	if (txt->length != 0) {
		memmove(&passwd[txt->cur_insert + txt->length], &passwd[txt->cur_insert], plen - txt->cur_insert + 1);
		memcpy(&passwd[txt->cur_insert], txt->text, txt->length);
	}

	// If any new characters are involved, set them to be '*'. The text
	// widget will now see and echo the input characters as such.
	if (txt->text != NULL)
		memset(txt->text, '*', txt->length);

	return(Pt_CONTINUE);
}
