/* Empfangen von Daten fr das KonnMuck-Photon-Programm

									Datei: receive.c
									Datum: 01.09.1996
									Rev. : 11.12.1996
									Autor: KS

rcs_id: $Id: receive.c,v 4.0 1997/01/04 12:25:59 klaus Exp klaus $

*/

//#define DEBUG

#include <stdio.h>      /* printf() */
#include <string.h>     /* strcmp(), strcpy() */
#include <stdlib.h>     /* read() */
#include <sys/kernel.h> /* Send(), Receive(), Reply() */
#include <errno.h>

#define __RECEIVE_PROC__
#include "header.h"


/* Konstanten */
#define FALSCHE_KENNUNG "Fehler!!"
#define FALSCHE_KENNUNG_LEN 9


void main(void)
{
	MSG_MELDUNG meldung;
	STRING_MELDUNG *strmsg;
	MSG_MELDUNG *msgmsg;
	pid_t pid;
	int sock;
	int len;

	/* Erst mal sehen, ob auch richtig aufgerufen */
	pid = Receive(0,&meldung,sizeof(MSG_MELDUNG));

	if (!KENNUNG_OKAY(&meldung)) {
		/* Falsche Kennung !! */
		Reply(pid,FALSCHE_KENNUNG,FALSCHE_KENNUNG_LEN);
		exit(-1);
	}

	/* War richtig! */
	
	/* Nur ausprobieren, ob das Programm verfgbar ist? */
	if (meldung.typ == KM_TEST) {
		meldung.typ = KM_IST_OKAY;
		Reply(pid,&meldung,sizeof(MELDUNG));
		exit(0);
	}
	
	/* Progamm benutzen */

	/* Puffer allokieren */
	if ((strmsg = (STRING_MELDUNG *)malloc(sizeof(STRING_MELDUNG)+BUFLEN)) == NULL) {
		meldung.typ = KM_FEHLER;
		Reply(pid,&meldung,sizeof(MELDUNG));
		exit(-2);
	}
	INIT_MELDUNG(strmsg,KM_STRING); /* Meldungsstruktur initialisieren */

	/* Alles Ok !! Los geht's */
	sock = meldung.val; /* Socket holen */

	meldung.typ = KM_IST_OKAY;  /* Antwort: OKAY */
	Reply(pid,&meldung,sizeof(MELDUNG));

	/* Eingabeschleife */
	while(sock) {

		#ifdef DEBUG
		printf("receive: Lese von Socket %d...\n",sock);
		#endif

		if ((len = read(sock,strmsg->string,BUFLEN-1)) == -1) {
			/* Ein Fehler ist aufgetreten */

			#ifdef DEBUG
			printf("receive: Fehler beim Lesen: %s\n",strerror(errno));
			#endif

			msgmsg = (MSG_MELDUNG *)strmsg;
			msgmsg->typ = KM_FEHLER;
			msgmsg->val = IV_F_READ;
			msgmsg->v_errno = errno;
			Send(pid,msgmsg,&meldung,sizeof(MSG_MELDUNG),sizeof(MELDUNG));
			free(strmsg);
			/* Beenden */
			exit(-1);
		}

		/* Es hat geklappt !! */

		if (len == 0) {
			/* Verbindung wahrscheinlich beendet */
			strmsg->typ = KM_CLOSESOCK;
			close(sock);
			sock = 0;
		}
		else {
			/* Etwas empfangen */
			*(strmsg->string+len) = '\0';
			strmsg->typ = KM_STRING;
		}

		#ifdef DEBUG
		printf("receive: %d Bytes empfangen.\n",len);
		printf("receive: Inhalt:\n");
		printf(strmsg->string);
		#endif

		#ifdef DEBUG
		printf("receive: Meldung schicken, Kennung = %s, typ = %d\n",strmsg->kennung,strmsg->typ);
		#endif

		Send(pid,strmsg,&meldung,sizeof(STRING_MELDUNG)+len+1,sizeof(MELDUNG));

		#ifdef DEBUG
		printf("receive: Antwort: Kennung = %s, Typ = %d\n",meldung.kennung,meldung.typ);
		#endif

		if (!OKAY(&meldung)) {
			free(strmsg);
			exit(-1);
		}
	} /* Von while(sock) */

	/* Verbindung beendet */
	free(strmsg);
	return;
}
